OK = 200
RESOURCE_CREATED = 201
ACCEPTED = 202
NO_CONTENT = 204
RESET_CONTENT = 205
PARTIAL_CONTENT = 206
FILE_UPLOADED = 226
MOVED_PERMANENTLY = 301
BAD_REQUEST = 400
UNAUTHORIZED = 401
FORBIDDEN = 403
NOT_FOUND = 404
METHOD_NOT_ALLOWED = 405
NOT_ACCEPTABLE = 406
CONFLICT = 409
SESSION_EXPIRED = 419
UNSUPPORTED_MEDIA_TYPE = 4015
INTERNAL_SERVER_ERROR = 500
NOT_IMPLEMENTED = 501
